@@ -549,7 +549,6 @@ class EventManager:
             debug.println(debug.LEVEL_WARNING, msg, True)
         else:
             skipCheck = [
-                "object:active-descendant-changed",
                 "object:children-changed",
                 "object:column-reordered",
                 "object:row-reordered",
@@ -652,7 +651,7 @@ class EventManager:
                 debug.printException(debug.LEVEL_WARNING)
                 return
 
-        if eType.startswith("window"):
+        if eType.startswith("window:") and not eType.endswith("create"):
             _scriptManager.reclaimScripts()
 
         if eType.startswith("object:state-changed:active"):
@@ -709,6 +708,9 @@ class EventManager:
             return
 
         setNewActiveScript, reason = self._isActivatableEvent(event, script)
+        msg = 'EVENT MANAGER: Change active script: %s (%s)' % (setNewActiveScript, reason)
+        debug.println(debug.LEVEL_INFO, msg, True)
+
         if setNewActiveScript:
             try:
                 app = event.host_application or event.source.getApplication()
@@ -730,6 +732,18 @@ class EventManager:
             debug.println(debug.LEVEL_INFO, msg, True)
             debug.printException(debug.LEVEL_INFO)
 
+        msg = 'EVENT MANAGER: locusOfFocus: %s activeScript: %s' % \
+              (orca_state.locusOfFocus, orca_state.activeScript)
+        debug.println(debug.LEVEL_INFO, msg, True)
+
+        if not orca_state.activeScript:
+            return
+
+        attributes = orca_state.activeScript.getTransferableAttributes()
+        for key, value in attributes.items():
+            msg = 'EVENT MANAGER: %s: %s' % (key, value)
+            debug.println(debug.LEVEL_INFO, msg, True)
+
     def _processKeyboardEvent(self, event):
         keyboardEvent = input_event.KeyboardEvent(event)
         if not keyboardEvent.is_duplicate:
