@@ -87,13 +87,15 @@ class LabelInference:
         if not result:
             result, objects = obj.name, []
             debug.println(debug.LEVEL_FINE, "INFER - Name: %s" % result)
-        if not result:
-            result, objects = obj.description, []
-            debug.println(debug.LEVEL_FINE, "INFER - Description: %s" % result)
         if result:
             result = result.strip()
             result = result.replace("\n", " ")
 
+        # Desperate times call for desperate measures....
+        if not result:
+            result, objects = self.inferFromTextLeft(obj, proximity=200)
+            debug.println(debug.LEVEL_FINE, "INFER - Text Left with proximity of 200: %s" % result)
+
         self.clearCache()
         return result, objects
 
@@ -187,7 +189,7 @@ class LabelInference:
             return False
 
         rv = self._isWidgetCache.get(hash(obj))
-        if rv != None:
+        if rv is not None:
             return rv
 
         widgetRoles = [pyatspi.ROLE_CHECK_BOX,
@@ -203,6 +205,9 @@ class LabelInference:
                        pyatspi.ROLE_PUSH_BUTTON]
 
         isWidget = obj.getRole() in widgetRoles
+        if not isWidget and obj.getState().contains(pyatspi.STATE_EDITABLE):
+            isWidget = True
+
         self._isWidgetCache[hash(obj)] = isWidget
         return isWidget
 
@@ -401,6 +406,9 @@ class LabelInference:
             return None, []
 
         prevObj, start, end, string = prevLine[0]
+        if self._cannotLabel(prevObj):
+            return None, []
+
         if string.strip():
             x, y, width, height = self._getExtents(prevObj, start, end)
             objX, objY, objWidth, objHeight = self._getExtents(obj)
@@ -441,6 +449,9 @@ class LabelInference:
             return None, []
 
         nextObj, start, end, string = nextLine[0]
+        if self._cannotLabel(nextObj):
+            return None, []
+
         if string.strip():
             x, y, width, height = self._getExtents(nextObj, start, end)
             objX, objY, objWidth, objHeight = self._getExtents(obj)
@@ -533,7 +544,7 @@ class LabelInference:
             return None, []
 
         cells = [table.getAccessibleAt(i, col) for i in range(1, table.nRows)]
-        cells = [x for x in cells if x != None]
+        cells = [x for x in cells if x is not None]
         if [x for x in cells if x.childCount and x[0].getRole() != obj.getRole()]:
             return None, []
 
