@@ -294,7 +294,7 @@ class ScriptManager:
         except:
             forceAppScript = False
         else:
-            forceAppScript = role == pyatspi.ROLE_FRAME
+            forceAppScript = role in [pyatspi.ROLE_FRAME, pyatspi.ROLE_STATUS_BAR]
 
         # Only defer to the toolkit script for this object if the app script
         # is based on a different toolkit.
