@@ -840,6 +840,12 @@ class Script(default.Script):
         if self._lastCommandWasCaretNav or args.get("includeContext"):
             priorObj, priorOffset = self.utilities.getPriorContext()
 
+        if obj.getRole() == pyatspi.ROLE_ENTRY:
+            utterances = self.speechGenerator.generateSpeech(obj, priorObj=priorObj)
+            speech.speak(utterances)
+            self.updateBraille(obj)
+            return
+
         offset = args.get("offset", 0)
         contents = self.utilities.getObjectContentsAtOffset(obj, offset)
         self.displayContents(contents)
@@ -884,7 +890,8 @@ class Script(default.Script):
         obj, offset = self.utilities.getCaretContext(documentFrame=None)
         if offset > 0 and isContentEditable:
             text = self.utilities.queryNonEmptyText(obj)
-            offset = min(offset, text.characterCount)
+            if text:
+                offset = min(offset, text.characterCount)
 
         contents = self.utilities.getLineContentsAtOffset(obj, offset)
         self.displayContents(contents)
@@ -1362,7 +1369,8 @@ class Script(default.Script):
             msg = "WEB: Event handled: Last command was mouse button"
             debug.println(debug.LEVEL_INFO, msg, True)
             self.utilities.setCaretContext(event.source, event.detail1)
-            orca.setLocusOfFocus(event, event.source)
+            notify = not self.utilities.isEntryDescendant(event.source)
+            orca.setLocusOfFocus(event, event.source, notify)
             return True
 
         if self.utilities.inFindToolbar():
@@ -1409,7 +1417,8 @@ class Script(default.Script):
             msg = "WEB: In content editable with embedded objects"
             debug.println(debug.LEVEL_INFO, msg, True)
             self.utilities.setCaretContext(obj, offset)
-            notify = not self.utilities.lastInputEventWasCharNav()
+            notify = not self.utilities.lastInputEventWasCharNav() \
+                     and not self.utilities.isEntryDescendant(obj)
             orca.setLocusOfFocus(event, event.source, notify)
             return False
 
@@ -1774,8 +1783,10 @@ class Script(default.Script):
             return True
 
         if self.utilities.eventIsChromePageSwitchNoise(event):
-            msg = "WEB: Ignoring event believed to be chrome page switch noise"
+            msg = "WEB: Event believed to be chrome page switch"
             debug.println(debug.LEVEL_INFO, msg, True)
+            if event.detail1:
+                self.presentObject(event.source)
             return True
 
         if not self.utilities.inDocumentContent(event.source):
