@@ -65,8 +65,8 @@ else:
     else:
         _speechd_version_ok = True
 
-PUNCTUATION = re.compile('[^\w\s]', re.UNICODE)
-ELLIPSIS = re.compile('(\342\200\246|(?<!\.)\.{3,4}(?=(\s|\Z)))')
+PUNCTUATION = re.compile(r'[^\w\s]', re.UNICODE)
+ELLIPSIS = re.compile('(\342\200\246|(?<!\\.)\\.{3,4}(?=(\\s|\\Z)))')
 
 class SpeechServer(speechserver.SpeechServer):
     # See the parent class for documentation.
@@ -100,7 +100,7 @@ class SpeechServer(speechserver.SpeechServer):
         """
         if serverId not in cls._active_servers:
             cls(serverId)
-        # Don't return the instance, unless it is succesfully added
+        # Don't return the instance, unless it is successfully added
         # to `_active_Servers'.
         return cls._active_servers.get(serverId)
 
